import ctypes
import ctypes.util
import logging
import sys

import psutil

from chouti.config import LD_PRELOAD, NO_MALLOC_TRIM

psutil_p = psutil.Process()

logger = logging.getLogger(__name__)

def release_mem():
    """
    workround for https://github.com/python/cpython/issues/84904
    not sure if it's useful
    """

    print(f"rss memoray usage: {psutil_p.memory_info().rss / 1024 / 1024:.2f}MB {LD_PRELOAD}")
    if psutil_p.memory_info().rss / 1024 / 1024 > 300:
            print("Memory usage > 300MB, memory leak? pls report! or restart the program.")
            print("use other malloc library may(?) help, like tcmalloc/mimalloc/jemalloc")

    # no action needed for mimalloc/jemalloc
    if NO_MALLOC_TRIM or 'mimalloc' in LD_PRELOAD or 'jemalloc' in LD_PRELOAD:
        logger.info("NO_MALLOC_TRIM or mimalloc/jemalloc in LD_PRELOAD, skip malloc_trim")
        return

    # call MallocExtension_ReleaseFreeMemory for tcmalloc
    if 'tcmalloc' in LD_PRELOAD:
        lib_path = ctypes.util.find_library("tcmalloc") or ctypes.util.find_library("tcmalloc_minimal")
        if not lib_path:
            print("libtcmalloc.so or libtcmalloc_minimal.so not found")
            return
        logger.debug("call MallocExtension_ReleaseFreeMemory") 
        ctypes.CDLL(lib_path).MallocExtension_ReleaseFreeMemory()
        return

    # libc malloc_trim
    try:
        ctypes.CDLL("libc.so.6").malloc_trim(0)
        print("malloc_trim done")
        print(f"rss memoray usage: {psutil_p.memory_info().rss / 1024 / 1024:.2f}MB")
    except Exception as e:
        print(f"malloc_trim FAIL: {type(e)} : {e}")
        if "win" in sys.platform:
            print("Windows? Don't worry about it.")
