import asyncio
import gzip
import json
import httpx
from httpx._content import encode_urlencoded_data

async def main():
    async with httpx.AsyncClient(http2=True, http1=True) as client:

        payload = {
            "title": "2021-07-26-1",
            "url": "https://www.chouti.com",
            "source": "chouti",
            "category": "news",
            "tags": ["news", "chouti"],
            "content": "This is a test content",
            "publish_time": 1627296000,
            "author": "yzqzss",
            "author_id": 0,
            "author_type": "int",
            "author_url": [
                "https://www.chouti.com/yzqzss",
                "https://www.chouti.com/y1111"
            ]
        }

        # /v1/project/:identifier/:client_version/:archivist/insert_item/:item_id
        data = {
                'item_id_type': "int",
                'item_status': None,
                'item_status_type': "None",
                'payload': json.dumps(payload, ensure_ascii=False, separators=(',', ':'))
            }

        _, encoded_data_stream = encode_urlencoded_data(data)
        encoded_data = b''.join(encoded_data_stream)
        compressed_data = gzip.compress(encoded_data)

        rate = len(compressed_data) / len(encoded_data)

        if rate > 0.9:
            r = await client.post("http://localhost:8080/v1/project/chouti_links/1.2/yzqzss-compression/insert_item/0",data=data)
        else:
            gzip_headers = { "Content-Encoding": "gzip", "Content-Type": "application/x-www-form-urlencoded"}
            r = await client.post("http://localhost:8080/v1/project/chouti_links/1.2/yzqzss-compression/insert_item/0",content=compressed_data, headers=gzip_headers)
        
        print(r.text)
        print(len(r.request.content))
        print(r.request.headers)
        print(r.headers)

if __name__ == "__main__":
    asyncio.run(main())